from .advanced.reducto import ReductoAdapter
from .base import Adapter, HTMLAdapter, JSONAdapter, PDFAdapter, TextAdapter

__all__ = [
    "Adapter",
    "PDFAdapter",
    "TextAdapter",
    "JSONAdapter",
    "HTMLAdapter",
    "ReductoAdapter",
]
